#if !defined(BIBOB_VIEW_INCLUDED_)
#define BIBOB_VIEW_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class CBibobView : public CView
{
protected: // create from serialization only
	CBibobView();
	DECLARE_DYNCREATE(CBibobView)

public:
	virtual ~CBibobView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	CBibobDoc* GetDocument();
	CChildFrame* getFrame() const;
	u64 getCurBB() {return GetDocument()->getBitBoard();}
	const CString& getFen() {return GetDocument()->getFen();}
	bool hasFenInternal() {return GetDocument()->hasFen();}
	bool hasFen();
	u64 getSecBB() {return getFrame()->getSecondOperand();}
	bool hasSecBB() {return getFrame()->hasSecondOperand();}

	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreTranslateMessage(MSG* pMsg);

protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);

	void GetRect4Board(CRect &rect) const;
	void getBoardRect(CRect &rect) const;
	void getTextRect(CRect &rect) const;
	void getTextRect1(CRect &rect) const;
	void getFenRect(CRect &rect) const;
	void getSquareRect(int sq, CRect &rect) const;
	int  getLineHeight() const;
	
	void drawSquare(CDC* pDC, int sq, bool isone);
	void drawBitBoard(CDC* pDC, u64 bb);
	void drawBitBoardAsString(CDC* pDC, u64 bb);
	CFont* selectFont(CDC* pDC);

	int  findSquareByPoint(CPoint point) const;
	void invalidate(u64 changed);
	void invalidateSquare(int sq);
	void invalidateText();
	void squareUpdate(int sq);
	void toggleSquare(int sq);
	void setBitboard(u64 bb);
	bool something2paste();
	bool hasFenInClipBoard();
	u32	 fenOptionMask() const;
	void paste(u32 mask);
	void pasteFen(u32 mask);

protected:
	CFont m_Font;
	enum {none, all, white, black} m_fenOption;
	int  m_CurrentSquare;
	u32  m_pmask;

protected:
	static COLORREF m_sSquareColors[2];
	static bool m_sHasFen;
	static bool m_sSomething2Paste;

protected:
	afx_msg void OnEditCopy();
	afx_msg void OnEditPaste();
	afx_msg void OnEditCut();
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnClear();
	afx_msg void OnSetall();
	afx_msg void OnDecrement();
	afx_msg void OnIncrement();
	afx_msg void OnInvert();
	afx_msg void OnMinus();
	afx_msg void OnShl();
	afx_msg void OnShr();
	afx_msg void OnDown();
	afx_msg void OnUp();
	afx_msg void OnAssign();
	afx_msg void OnAdd();
	afx_msg void OnSub();
	afx_msg void OnMul();
	afx_msg void OnMulh();
	afx_msg void OnDiv();
	afx_msg void OnModulus();
	afx_msg void OnAnd();
	afx_msg void OnOr();
	afx_msg void OnXor();
	afx_msg void OnAllFen();
	afx_msg void OnUpdateAllFen(CCmdUI* pCmdUI);
	afx_msg void OnBlackFen();
	afx_msg void OnUpdateBlackFen(CCmdUI* pCmdUI);
	afx_msg void OnWhiteFen();
	afx_msg void OnUpdateWhiteFen(CCmdUI* pCmdUI);
	afx_msg void OnKing();
	afx_msg void OnUpdateKing(CCmdUI* pCmdUI);
	afx_msg void OnKnight();
	afx_msg void OnUpdateKnight(CCmdUI* pCmdUI);
	afx_msg void OnPawn();
	afx_msg void OnQueen();
	afx_msg void OnUpdateQueen(CCmdUI* pCmdUI);
	afx_msg void OnUpdatePawn(CCmdUI* pCmdUI);
	afx_msg void OnRook();
	afx_msg void OnUpdateRook(CCmdUI* pCmdUI);
	afx_msg void OnBishop();
	afx_msg void OnUpdateBishop(CCmdUI* pCmdUI);
	afx_msg void OnFenNoPieces();
	afx_msg void OnUpdateFenNoPieces(CCmdUI* pCmdUI);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnFenChoosed();
	afx_msg void OnAndNot();
	afx_msg void OnInt2float();
	afx_msg void OnFloat2int();
	afx_msg void On1DivN();
	afx_msg void OnMirror();
	afx_msg void OnFlip();
	afx_msg void OnDiaFlip();
	afx_msg void OnAntFlip();
	afx_msg void OnReverse();
	afx_msg void OnAddToCombo();
	afx_msg void OnUpdateAddToCombo(CCmdUI* pCmdUI);
	afx_msg void OnAddFloatToCombo();
	afx_msg void OnUpdateAddFloatToCombo(CCmdUI* pCmdUI);
	afx_msg void OnStop();
	afx_msg void OnE();
	afx_msg void OnPi();
	DECLARE_MESSAGE_MAP()
};


#ifndef _DEBUG  // debug version in bbviewView.cpp
inline CBibobDoc* CBibobView::GetDocument()
   { return (CBibobDoc*)m_pDocument; }
#endif


#endif // !defined(BIBOB_VIEW_INCLUDED_)
